unit ServerController;

interface

uses
  SysUtils, Classes, IWServerControllerBase, IWBaseForm, HTTPApp,
  // For OnNewSession Event
  UserSessionUnit, IWApplication, IWAppForm, WideStrings, DBXFirebird, DB,
  SqlExpr, ExtCtrls, FMTBcd, IdBaseComponent, IdComponent, IdTCPConnection,
  IdTCPClient, IdExplicitTLSClientServerBase, IdMessageClient, IdSMTPBase,
  IdSMTP, IdMessage;

type
  TIWServerController = class(TIWServerControllerBase)
    timerAlerta: TTimer;
    sqlConnAlerta: TSQLConnection;
    qryAlerta: TSQLQuery;
    IdSMTP: TIdSMTP;
    IdMessage: TIdMessage;
    qryAlertaCODAGE: TIntegerField;
    qryAlertaNOMTIP: TStringField;
    qryAlertaNOMUSU: TStringField;
    qryAlertaEMAUSU: TStringField;
    qryAlertaINIAGE: TSQLTimeStampField;
    qryAlertaASSAGE: TStringField;
    procedure IWServerControllerBaseNewSession(ASession: TIWApplication;
      var VMainForm: TIWBaseForm);
    procedure timerAlertaTimer(Sender: TObject);

  private
    function EnviarEmail(_destinatario, _assunto, _corpo: String): Boolean;

  public
  end;

function UserSession: TIWUserSession;
function IWServerController: TIWServerController;

implementation

{$R *.dfm}

uses
  IWInit, IWGlobal, uLogin;

function IWServerController: TIWServerController;
begin
  Result := TIWServerController(GServerController);
end;

function UserSession: TIWUserSession;
begin
  Result := TIWUserSession(WebApplication.Data);
end;

procedure TIWServerController.IWServerControllerBaseNewSession
  (ASession: TIWApplication; var VMainForm: TIWBaseForm);
begin
  ASession.Data := TIWUserSession.Create(nil);
end;

function TIWServerController.EnviarEmail(_destinatario, _assunto,
  _corpo: String): Boolean;
begin
  Result := False;

  IdMessage.Clear;
  IdMessage.From.Address := 'kelver@activedelphi.com.br';
  IdMessage.From.Name := 'Agenda On-Line - ActiveDelphi.com.br';
  IdMessage.Recipients.EMailAddresses := _destinatario;
  IdMessage.Priority := mpHigh;
  IdMessage.Subject := _assunto;
  IdMessage.ContentType := 'text/html';
  IdMessage.Body.Text := _corpo;

  try
    try
      if not IdSMTP.Connected then
        IdSMTP.Connect;
      IdSMTP.Authenticate;
      IdSMTP.Send(IdMessage);
      Result := True;
    except
      Result := False;
    end
  finally
    try
      IdSMTP.Disconnect;
    except
    end;
  end;
end;

procedure TIWServerController.timerAlertaTimer(Sender: TObject);
var
  ema, ass, msg: String;
begin
  //desativa o timer assim que entra no evento, para no correr o
  //risco que o timer dispare novamente antes da execuo anterior
  //terminar.
  timerAlerta.Enabled := False;
  try

    //fecha a query e abre novamente
    qryAlerta.Close;
    qryAlerta.Open;

    //repassa os agendamentos que precisam ser alertados e que ainda
    //no foi enviado e-mail
    while (not qryAlerta.IsEmpty) and (not qryAlerta.Eof) do
    begin

      //recupera o e-mail do usurio, monta o assunto e o corpo do e-mail
      ema := qryAlertaEMAUSU.Value;
      ass := 'Lembrete Agenda On-Line: ' + qryAlertaNOMTIP.AsString;
      msg := '<p>Caro(a) <b>%s</b>:<br><br>' +
             'Seu compromisso "<b>%s</b>" esta agendado para <b>%s</b>.<br>' +
             'Segue abaixo a descrio do agendamento:<br><br><blockquote>' +
             '%s</blockquote><br>Atenciosamente,<br>Agenda On-Line<br>' +
             '<a href="http://www.activedelphi.com.br" target="_blank" ' +
             'alt="ActiveDelphi.com.br"><img src="http://www.activedelphi.' +
             'com.br/images/logo_active.gif" border=0 /></a></p>';

      //troca os %s da varivel msg com seus respectivos valores, para
      //montar o cdigo html corretamente
      msg := Format(msg,
                    [qryAlertaNOMUSU.Value,
                    qryAlertaNOMTIP.Value,
                    FormatDateTime('DD/MM/YYYY" s "HH:NN" hora(s)"',
                                   qryAlertaINIAGE.AsDateTime),
                    qryAlertaASSAGE.Value]);

      //aps montar a preparao do e-mail, chama a funo que o envia
      //e se deu tudo certo, atualiza o agendamento em questo marcando-o
      //como j enviado alerta
      if EnviarEmail(ema, ass, msg) then
        sqlConnAlerta.ExecuteDirect('UPDATE AGENDA SET ENVIOU = ''S'' WHERE ' +
                                    'CODAGE = ' + qryAlertaCODAGE.AsString);

      //vai para o prximo agendamento que precisa ser alertado
      qryAlerta.Next;
    end;
    qryAlerta.Close;

  finally
    //reativa o timer no final do procedimento, para que daqui a 1 minuto,
    //esta mesma verificao volte a ser realizada
    timerAlerta.Enabled := True;
  end;
end;

initialization

TIWServerController.SetServerControllerClass;

end.
